@@ -5,6 +5,8 @@ use strict;
 use Carp;
 use File::Spec;
 use Module::Info;
+use Scalar::Util;
+
 use Genealogy::Wills::wills;
 
 =head1 NAME
@@ -13,11 +15,11 @@ Genealogy::Wills - Lookup in a database of wills
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -32,29 +34,53 @@ our $VERSION = '0.05';
 
 Creates a Genealogy::Wills object.
 
-Takes two optional arguments:
-	directory: that is the directory containing obituaries.sql
-	logger: an object to send log messages to
+Takes two optional arguments which can be hash, hash-ref or key-value pairs.
+
+=over 4
+
+=item C<directory>
+
+That is the directory containing obituaries.sql.
+If not given, the use the module's data directory.
+
+=item C<logger>
+
+An object to send log messages to
+
+=back
 
 =cut
 
-sub new {
+sub new
+{
 	my $class = shift;
-	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+
+	# Handle hash or hashref arguments
+	my %args;
+	if((@_ == 1) && (ref($_[0]) eq 'HASH')) {
+		%args = %{$_[0]};
+	} elsif((@_ % 2) == 0) {
+		%args = @_;
+	} else {
+		Carp::croak(__PACKAGE__, ': Invalid arguments passed to new()');
+		return;
+	}
 
 	if(!defined($class)) {
-		# Using Genealogy::Wills->new(), not Genealogy::Wills::new()
-		# carp(__PACKAGE__, ' use ->new() not ::new() to instantiate');
-		# return;
+		if((scalar keys %args) > 0) {
+			# Using Genealogy::Wills::new(), not Genealogy::Wills->new()
+			carp(__PACKAGE__, ' use ->new() not ::new() to instantiate');
+			return;
+		}
 
 		# FIXME: this only works when no arguments are given
 		$class = __PACKAGE__;
-	} elsif(ref($class)) {
+	} elsif(Scalar::Util::blessed($class)) {
 		# clone the given object
 		return bless { %{$class}, %args }, ref($class);
 	}
 
-	if(!defined((my $directory = ($args{'directory'} || $Database::Abstraction::init->{'directory'})))) {
+	if(!defined((my $directory = ($args{'directory'} || $Database::Abstraction->{'directory'})))) {
 		# If the directory argument isn't given, see if we can find the data
 		$directory ||= Module::Info->new_from_loaded(__PACKAGE__)->file();
 		$directory =~ s/\.pm$//;
@@ -74,6 +100,13 @@ sub new {
 
 =head2 search
 
+Last (last name) is a mandatory parameter.
+
+Return a list of hash references in list context,
+or a hash reference in scalar context.
+
+Each record includes a formatted C<url> field.
+
     my $wills = Genealogy::Wills->new();
 
     # Returns an array of hashrefs
@@ -85,10 +118,9 @@ sub new {
 
 sub search {
 	my $self = shift;
+	my $params = $self->_get_params('last', @_);
 
-	my %params = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
-
-	if(!defined($params{'last'})) {
+	if(!defined($params->{'last'})) {
 		Carp::carp("Value for 'last' is mandatory");
 		return;
 	}
@@ -100,17 +132,52 @@ sub search {
 	}
 
 	if(wantarray) {
-		my @wills = @{$self->{'wills'}->selectall_hashref(\%params)};
+		my @wills = @{$self->{'wills'}->selectall_hashref($params)};
 		foreach my $will(@wills) {
 			$will->{'url'} = 'https://' . $will->{'url'};
 		}
 		return @wills;
 	}
-	my $will = $self->{'wills'}->fetchrow_hashref(\%params);
+	my $will = $self->{'wills'}->fetchrow_hashref($params);
 	$will->{'url'} = 'https://' . $will->{'url'};
 	return $will;
 }
 
+# Helper routine to parse the arguments given to a function.
+# Processes arguments passed to methods and ensures they are in a usable format,
+#	allowing the caller to call the function in anyway that they want
+#	e.g. foo('bar'), foo(arg => 'bar'), foo({ arg => 'bar' }) all mean the same
+#	when called _get_params('arg', @_);
+sub _get_params
+{
+	shift;  # Discard the first argument (typically $self)
+	my $default = shift;
+
+	# Directly return hash reference if the first parameter is a hash reference
+	return $_[0] if(ref $_[0] eq 'HASH');
+
+	my %rc;
+	my $num_args = scalar @_;
+
+	# Populate %rc based on the number and type of arguments
+	if(($num_args == 1) && (defined $default)) {
+		# %rc = ($default => shift);
+		return { $default => shift };
+	} elsif($num_args == 1) {
+		Carp::croak('Usage: ', __PACKAGE__, '->', (caller(1))[3], '()');
+	} elsif(($num_args == 0) && (defined($default))) {
+		Carp::croak('Usage: ', __PACKAGE__, '->', (caller(1))[3], "($default => \$val)");
+	} elsif(($num_args % 2) == 0) {
+		%rc = @_;
+	} elsif($num_args == 0) {
+		return;
+	} else {
+		Carp::croak('Usage: ', __PACKAGE__, '->', (caller(1))[3], '()');
+	}
+
+	return \%rc;
+}
+
 =head1 AUTHOR
 
 Nigel Horne, C<< <njh at bandsman.co.uk> >>
@@ -151,7 +218,7 @@ L<http://deps.cpantesters.org/?module=Genealogy::Wills>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2023-2024 Nigel Horne.
+Copyright 2023-2025 Nigel Horne.
 
 This program is released under the following licence: GPL2
 
